#!/bin/bash

# Copyright (c) 2002-2007 Userful Corporation. All rights reserved.
# http://www.userful.com/

# install-license.sh - get some info from the system, submit it via URL, and 
# if we get a valid license: install it, if not: display a meaningful
# error message

function usage() {
    echo "
This script attempts to install a license key from store.userful.com.

Options:
   -a code      supply an Activation code
   -f file      supply a File that contains an activation code
   -d           Delete the activation file after using it (happens even
                if there is an error)
   -D           only Delete the file if a license was successfully installed
                if there is an error)
   -n           do Not use the graphical display 
   -h           display this Help message and exit
   -s #         specify the number of Stations to request 
   -l file      install a local file - can be a zip file or the actual license file
"
}

TEXTDOMAINDIR=/usr/share/locale
TEXTDOMAIN=userful-about-dialog

ACTIVATION_CODE=""
ACTIVATION_FILE=""
ACTIVATION_CODE_FROM_FILE=""
DELETE_ACTIVATION_FILE=false
ACTIVATION_CODE=""
SUPPLIED_ACTIVATION_FILE=false
NON_GRAPHICAL=false
DELETE_ACTIVATION_FILE_ON_SUCCESS=false
NUMBER_OF_STATIONS=0

PROVIDED_LICENSE=""

# parse arguments:
while getopts "a:df:hns:l:" flag ; do
    case "$flag" in
        a)
            ACTIVATION_CODE="$OPTARG"
            ;;
        d)
            DELETE_ACTIVATION_FILE=true
            ;;
        D)
            DELETE_ACTIVATION_FILE_ON_SUCCESS=true
            ;;
        f)
            SUPPLIED_ACTIVATION_FILE=true
            ACTIVATION_FILE="$OPTARG"
            ;;
        h)
            usage
            exit 0
            ;;
        n)
            NON_GRAPHICAL=true
            ;;
        s)
            NUMBER_OF_STATIONS="$OPTARG"
            ;;
        l)
            PROVIDED_LICENSE="$OPTARG"
            ;;
        *)
            usage
            exit 1
            ;;
    esac
done

###########################################################################
#
# DEFINES 
#
# define some basic stuff that will be used throughout the file
#
###########################################################################

# MAC ADDRESS (in various forms) --- #### THIS WILL BE REPLACED BY THE A UTILITY TO LOOK AT ALL INTERFACES #####
ETH_NUM=0
MAC=`LANG=C /sbin/ifconfig -a eth$ETH_NUM | grep "HWaddr" | awk '{ print $5 }'`
if [ -z "$MAC" ] || [ ${#MAC} != 17 ] ; then 
    # MAC is no good
    MAC=nomac
fi
MAC_COLONS=$MAC
MAC_DASHES=${MAC//:/-}
MAC_UNDERS=${MAC//:/_}

# 1Box Utils' SYS-ID (not to be confused with DiscoverStation's systemid)
SYS_ID=unknown
if [ -f /etc/userful/sys-id ] ; then
   SYS_ID_TMP=`cat /etc/userful/sys-id`
   if [ -n "$SYS_ID_TMP" ] ; then
       SYS_ID=$SYS_ID_TMP
   fi
fi
SYS_ID_PART=${SYS_ID#*-}


# various helper programs

# check to see is local versions of the programs exists, 
# if they do, use them, if not, use the default install locations
# if those don't exists use 'echo'
ERR_DLG=/usr/share/1box-utils/error-dialog.py
if [ ! -x $ERR_DLG ] ; then
    ERR_DLG=./error-dialog.py
fi 
if [ ! -x $ERR_DLG ] ; then
    ERR_DLG=echo
fi

ACTIVATION_DLG=./activation-dialog.py
if [ ! -x $ACTIVATION_DLG ] ; then
    ACTIVATION_DLG=/usr/share/1box-utils/activation-dialog.py
fi
if [ ! -x $ACTIVATION_DLG ] ; then
    ACTIVATION_DLG=$ERR_DLG
fi

PROGRESS_DLG=./progress-dialog.py
if [ ! -x $PROGRESS_DLG ] ; then
    PROGRESS_DLG=/usr/share/1box-utils/progress-dialog.py
fi
if [ ! -x $PROGRESS_DLG ] ; then
    PROGRESS_DLG=$ERR_DLG
fi

if [ -z "$DISPLAY" ] ; then
    ERR_DLG=echo
    ACTIVATION_DLG=echo
    PROGRESS_DLG=echo
fi

if [ $NON_GRAPHICAL == true ] ; then
    unset DISPLAY
fi

################
#
# END DEFINES
#
################

################
#
# Function: install_file
#
# takes the provided file and puts it in the right place
# - makes a backup if one already exists
# - attempts to get root privileges if not already
#
# TODO: check the priority of the file and warn the user if this file probably will not be used due to another file in a higher priority
#
################
function install_file () {

    SRC_THING=$1

    STAT=`which stat`
    RET=$?
    if [ $RET != 0 ] ; then
        STAT=false
    fi

    SIZE=`$STAT -c %s $SRC_THING`
    RET=$?
    if [ $RET == 0 ] ; then
        if [ "$SIZE" != 256 ] ; then
            # the size is *NOT* the correct size to be a license file
            exec $ERR_DLG $"The file $SRC_THING is not the correct size to be a license file."
        fi
    fi

    INSTALL_DIR=/etc/X11
    if [ ! -w $INSTALL_DIR ] ; then
        mv -f $SRC_THING /tmp/1Box.info &> /dev/null && SRC_THING=/tmp/1Box.info
        exec $ERR_DLG $"License key retrieved but not installed. You must be root for automatic installation. The file has been saved to $SRC_THING. Ask your system administrator to copy this file to $INSTALL_DIR or re-run this application as root."
    fi

    BACKUP_DIR=$INSTALL_DIR/1Box-info-backup
    mkdir -p $BACKUP_DIR 
    
    if ls $INSTALL_DIR/*1Box.info* &> /dev/null ; then
        mv -f --backup=numbered $INSTALL_DIR/*1Box.info* $BACKUP_DIR
    fi
    
    mv $SRC_THING $INSTALL_DIR/1Box.info 
    
    if [ "$DELETE_ACTIVATION_FILE_ON_SUCCESS" == true ] ; then
        if [ -n "$ACTIVATION_FILE" ] && [ -f $ACTIVATION_FILE ] ; then
            rm -f $ACTIVATION_FILE || exit 1
        fi
    fi
    
    MESSAGE=$"Successfully installed license file."
    # "sucess" spelt wrong intentionally
    $ERR_DLG type=sucess "text=$MESSAGE"
}

##############################
#
# LOCAL LICENSE FILE INSTALL
#
##############################
if [ -n "$PROVIDED_LICENSE" ] ; then

    ARCHIVE=no
    
    UNZIP_TMP=`mktemp -d -t install-license.sh.XXXXXX`
    trap "rm -rf $UNZIP_TMP 2>/dev/null" EXIT
    
    mkdir -p $UNZIP_TMP

    # this file would be passed from the activation dialog - so it should exist

    if [ $PROVIDED_LICENSE != "${PROVIDED_LICENSE%.zip}" ] ; then
        ARCHIVE=yes

        # the provided file looks to be a zip file

        cd $UNZIP_TMP

        unzip $PROVIDED_LICENSE -d $UNZIP_TMP
    fi

    if [ $PROVIDED_LICENSE != "${PROVIDED_LICENSE%.tar.bz2}" ] ; then
        ARCHIVE=yes

        # the provided file looks to be a bzip file

        cd $UNZIP_TMP

        tar jxf $PROVIDED_LICENSE
    fi
    if [ $PROVIDED_LICENSE != "${PROVIDED_LICENSE%.tar.gz}" -o $PROVIDED_LICENSE != "${PROVIDED_LICENSE%.tgz}" ] ; then
        ARCHIVE=yes

        # the provided file looks to be a qzip file

        cd $UNZIP_TMP

        tar zxf $PROVIDED_LICENSE
    fi


    if [ "$ARCHIVE" == yes ] ; then

        FOUND=`find $UNZIP_TMP -iname $MAC_DASHES -or -iname "$SYS_ID_PART"`

        STATUS=0
        if [ -n "$FOUND" ] ; then
            for F in $FOUND ; do 
                install_file $F                
            done
            
            exit 0
            
        else
            MESSAGE=$"Zip file ($PROVIDED_LICENSE) does not contain a license file with a matching MAC address."
            $ERR_DLG "text=Error: $MESSAGE"
            exit 1
        fi
    fi
    # file does not appear to be a zip file, check the size and maybe install it
    
    STAT=`which stat`
    RET=$?
    if [ $RET != 0 ] ; then
        STAT=true
    fi

    SIZE=`$STAT -c %s $PROVIDED_LICENSE`
    if [ "$SIZE" == 256 ] ; then
        # the size is the correct size to be a license file

        install_file $PROVIDED_LICENSE

        RET=$?
        if [ $RET != 0 ] ; then
            $ERR_DLG "text=Error: $OUT"
        fi
        exit $RET
    fi

    # Bug #666 - look for files that might work in the directory.

    CONTAINING_DIR=${PROVIDED_LICENSE%/*}

    FOUND=`find $CONTAINING_DIR -iname $MAC_DASHES -or -iname $SYS_ID_PART`
    if [ -n "$FOUND" ] ; then
        for FILE in $FOUND ; do
            install_file $PROVIDED_LICENSE
            RET=$?
            exit $RET
        done
    fi

    MESSAGE=$"The file ($PROVIDED_LICENSE) does not appear to be a valid license file. Please choose another file."
    $ERR_DLG "text=$MESSAGE"
    exec $ACTIVATION_DLG

fi

###############################################
#
# Deal with the activation code file
#
###############################################

if [ $DELETE_ACTIVATION_FILE == true ] && [ $SUPPLIED_ACTIVATION_FILE == false ] ; then
    echo "ERROR: specified activation code file deletion, but did not specify an activation code file"
    usage
    exit 1
fi

if [ $SUPPLIED_ACTIVATION_FILE == true ] ; then
    if [ -n "$ACTIVATION_FILE" ] && [ -f "$ACTIVATION_FILE" ] ; then
        ACTIVATION_CODE_FROM_FILE=`cat "$ACTIVATION_FILE"` || exit 1
        if [ -z "$ACTIVATION_CODE_FROM_FILE" ] ; then
            echo "ERROR: activation file \"$ACTIVATION_FILE\" is empty"
            usage
            exit 1
        fi
    else
        echo "ERROR: \"$ACTIVATION_FILE\" does not exist"
        usage
        exit 1
    fi

    if [ "$DELETE_ACTIVATION_FILE" == true ] ; then
        if [ -n "$ACTIVATION_FILE" ] && [ -f $ACTIVATION_FILE ] ; then
            rm -f $ACTIVATION_FILE || exit 1
        fi
    fi
fi

# at this point the non-getopt style arguments should remain - this is backward compatible with the way it used to work

#######################################
#
# Show a progress dialog
#
#######################################

$PROGRESS_DLG &
PROGRESS_DLG_PID=$!
KILL_PROGRESS="kill -SIGKILL $PROGRESS_DLG_PID"

###################################################
#
# Build the URL to get the license from the store
#
###################################################

URL="http://store.userful.com/license/control/GetLicense?"

SYSID=none
if [ -f /etc/userful/sys-id ] ; then
    SYSID=`cat /etc/userful/sys-id`
fi
URL="${URL}sysid=$SYSID"

# We always include an activation code even if it is empty because the 
# server requires it
if [ -n "$ACTIVATION_CODE_FROM_FILE" ] ; then
    if [ -n "$ACTIVATION_CODE" ] ; then
        echo "WARNING: both an activation code and activation code file were supplied, using activation code from file."
    fi
    ACTIVATION_CODE="$ACTIVATION_CODE_FROM_FILE"
fi
if [ -n "$ACTIVATION_CODE" ] ; then
    URL="${URL}&"
    URL="${URL}activation=$ACTIVATION_CODE"
fi


# license file order:
# http://wiki.userful.ca/wiki/index.php?title=License_File
# order of preferred MAC addr formats: : - _

FILENAME=
if [ -e /etc/X11/$MAC_COLONS.1Box.info ]; then
    FILENAME="/etc/X11/$MAC_COLONS.1Box.info"
elif [ -e /etc/X11/$MAC_DASHES.1Box.info ]; then
    FILENAME=/etc/X11/$MAC_DASHES.1Box.info
elif [ -e /etc/X11/$MAC_UNDERS.1Box.info ]; then
    FILENAME=/etc/X11/$MAC_UNDERS.1Box.info
elif [ -e /etc/X11/1Box.info ] ; then
    FILENAME="/etc/X11/1Box.info"
elif [ -e /etc/X11/1Box.info.trial ]; then
    FILENAME="/etc/X11/1Box.info.trial"
fi

LICENSE_KEY=
if [ -n "$FILENAME" ] ; then
    echo "Found existing license: $FILENAME"
    LICENSE_KEY="`/usr/share/1box-utils/b64encode.py $FILENAME`"
    URL="${URL}&"
    URL="${URL}lickey=$LICENSE_KEY"
else
    URL="${URL}&lickey="
fi

if [ $SYSID != none ] ; then
    MAC=000000000000
    # why not? 00000000 - Marvin says so
fi

if [ $MAC == nomac ] ; then
    MESSAGE=$"No valid MAC address found. Please contact Userful: http://store.userful.com/ or 1-(866)-USERFUL."
    exec $ERR_DLG "text=$MESSAGE"
else
    URL="${URL}&"
    URL="${URL}mac=${MAC//:/}"
fi

#if [ "$NUMBER_OF_STATIONS" == 0 ] ; then
#
#    X11_STATIONS=0
#
#    if [ -f /etc/X11/stations ] ; then
#        X11_STATIONS=`cat /etc/X11/stations`
#    fi
#
#    if [ "$X11_STATIONS" -gt $NUMBER_OF_STATIONS ] ; then
#        # this will also help detect if x11-stations is not a number
#        NUMBER_OF_STATIONS=$X11_STATIONS
#    fi
#fi

URL="${URL}&"
URL="${URL}stations=$NUMBER_OF_STATIONS"

# if an oem file exists, get the hash and send that too
OEMFILE=/etc/userful/oem
if [ -f $OEMFILE ] ; then
    which sha1sum &> /dev/null || OEMHASH=none
    if [ "$OEMHASH" != none ] ; then
        SHA=`sha1sum $OEMFILE`
        OEMHASH=${SHA// *}
        URL="${URL}&oem=$OEMHASH"
    fi
fi

# add the language to the url
if [ -n "$LANG" ] ; then
    URL="${URL}&"
	URL="${URL}lang=$LANG"
else
    # if it's empty, send it anyway
    URL="${URL}&"
	URL="${URL}lang="
fi

# add the version to the url
FILE=/etc/userful/dm-release
VERSION=unknown
if [ -f $FILE ] ; then
    OS_VERSION=`/usr/share/1box-utils/b64encode.py $FILE`
fi
URL="${URL}&"
URL="${URL}version=$OS_VERSION"

# add the OS version to the URL
FILE=/etc/userful/os-info
OS_VERSION="unknown=="
if [ -f $FILE ] ; then
    OS_VERSION=`/usr/share/1box-utils/b64encode.py $FILE`
fi
URL="${URL}&"
URL="${URL}osversion=$OS_VERSION"

TMP_DIR=/tmp/install-license-$$
mkdir -p $TMP_DIR

SRC_FILE=$TMP_DIR/1Box.info

echo "Attempting to get license file from $URL"

###################################
#
# Get license file
#
#################################3

C=`which curl`
W=`which wget`

RET=1
# we are no longer installing "get-url.py" because the required libraries are rarely installed
if [ -x /usr/share/1box-utils/get-url.py ] ; then
	/usr/share/1box-utils/get-url.py "$URL" >  $SRC_FILE 2> /dev/null
	RET=$?
fi
if [ $RET != 0 ] ; then
    if [ -n "$C" ] && [ -x $C ] ; then 
        curl "$URL" > $SRC_FILE 2> /dev/null
    elif [ -n "$W" ] && [ -x $W ] ; then 
        wget -O $SRC_FILE "$URL"
    else
        if [ -d /proc/$PROGRESS_DLG_PID ] ; then
            $KILL_PROGRESS
        fi
        exec $ERR_DLG "Could not use python urllib, find 'wget', or 'curl' to download license file. Go to http://store.userful.com/ to get your license."
    fi
fi

# to make the command line version look a little nicer
if [ -z "$DISPLAY" ] ; then
    echo
fi

#################################
#
# verify license file
#
###############################

SIZE=`stat -c "%s" $SRC_FILE`

if [ -d /proc/$PROGRESS_DLG_PID ] ; then
	$KILL_PROGRESS
fi

if [ $SIZE != 256 ] ; then
	# license is bad: it's actually an error message - show the message (or do something else)
    if [ -n "$ACTIVATION_CODE" ] ; then
        echo "${0##*/}: $ACTIVATION_DLG -a $ACTIVATION_CODE -t `cat $SRC_FILE`"
        exec $ACTIVATION_DLG -a "$ACTIVATION_CODE" -t "`cat $SRC_FILE`" 
    else
        if [ -z "$DISPLAY" ] ; then
            echo "We did not receive a license file from the server."
            echo "Please use go to http://store.userful.com/ to register. After registering, run this script again."
            echo "The text returned from the server was: "
        fi
        exec $ACTIVATION_DLG -t "`cat $SRC_FILE`" 
    fi
else
	:
	# license is good
fi

##########################################
#
# license is good: install license file
#
#########################################

install_file $SRC_FILE

exit 0
